@@ -12,6 +12,7 @@ import torch
 from abc import ABC
 from abc import abstractmethod
 from typing import Union
+from copy import deepcopy
 from sklearn.linear_model import LinearRegression
 from sklearn.linear_model import Lasso
 from sklearn.tree import DecisionTreeRegressor
@@ -104,6 +105,8 @@ class SKLearnModel(Model):
                  **kwargs):
         super(SKLearnModel, self).__init__(alg, SRC_SKLEARN)
         self.__model = None
+        self.__hparams = deepcopy(kwargs)
+
         self.__init_model(**kwargs)
 
     def __init_model(self,
@@ -149,6 +152,10 @@ class SKLearnModel(Model):
         else:
             raise AssertionError('Unknown Scikit-learn model \'{}\' was given.'.format(self.alg_id))
 
+    @property
+    def hparams(self):
+        return self.__hparams
+
     def fit(self,
             dataset: Union[Dataset, MultimodalDataset]) -> SKLearnModel:
         """
