@@ -1,11 +1,15 @@
 import os
 import json
+import numpy
 import torch
 from typing import Union
+from copy import deepcopy
 from ailca.core.env import *
 from ailca.data.base import Dataset
+from ailca.data.util import get_data_loader
 from ailca.data.multimodal import MultimodalDataset
 from ailca.ml.base import Model
+from ailca.ml.base import SKLearnModel
 from ailca.ml.base import PyTorchModel
 
 
@@ -149,3 +153,51 @@ def r2_score(targets: torch.Tensor,
     ss_res = torch.sum((targets - preds)**2)
 
     return (1 - ss_res / ss_tot).item()
+
+
+def k_fold_cross_validation(k_folds: list,
+                            model: Model,
+                            batch_size: int = 64,
+                            gradient_method: str = 'adam',
+                            loss_func: str = 'mae',
+                            n_epochs: int = 300,
+                            random_seed: int = 0):
+    preds = list()
+    scores = list()
+
+    for k in range(0, len(k_folds)):
+        dataset_train, dataset_test = k_folds[k]
+
+        if isinstance(model, SKLearnModel):
+            _model = deepcopy(model)
+            _model.fit(dataset_train)
+            preds_test = _model.predict(dataset_test)
+            preds.append(preds_test)
+            scores.append(r2_score(dataset_test.y, preds_test))
+        elif isinstance(model, PyTorchModel):
+            min_loss_val = 1e+8
+            dataset_train, dataset_val = dataset_train.split(ratio_train=0.8, random_seed=random_seed)
+            loader_train = get_data_loader(dataset_train, batch_size=batch_size, shuffle=True)
+            init_state_dict = deepcopy(model.state_dict())
+            best_state_dict = None
+
+            model.load_state_dict(deepcopy(init_state_dict))
+            optimizer = get_optimizer(model, gradient_method=gradient_method)
+            criterion = get_loss_func(loss_func=loss_func)
+
+            for n in range(0, n_epochs):
+                _ = model.fit(loader_train, optimizer, criterion)
+                preds_val = model.predict(dataset_val)
+                loss_val = criterion(preds_val, dataset_val.y)
+
+                if loss_val < min_loss_val:
+                    best_state_dict = deepcopy(model.state_dict())
+                    min_loss_val = loss_val
+
+            model.load_state_dict(best_state_dict)
+
+            preds_test = model.predict(dataset_test)
+            preds.append(preds_test)
+            scores.append(criterion(preds_test, dataset_test.y).item())
+
+    return torch.vstack(preds), numpy.mean(scores), numpy.std(scores)
